@@ -54,6 +54,15 @@ class Gencontrol(Base):
             Templates(template_dirs),
             VersionLinux)
         self.config_dirs = config_dirs
+
+        for debianrelease in self.config.debianreleases:
+            if debianrelease.name_regex.fullmatch(self.changelog[0].distribution):
+                self.debianrelease = debianrelease
+                break
+        else:
+            raise RuntimeError(
+                f'No debianrelease config matches {self.changelog[0].distribution}')
+
         self.process_changelog()
 
         for env, attr, desc in self.env_flags:
@@ -81,7 +90,7 @@ class Gencontrol(Base):
         super().do_main_setup(config, vars, makeflags)
         makeflags.update({
             'VERSION': self.version.linux_version,
-            'UPSTREAMVERSION': self.version.linux_upstream,
+            'UPSTREAMVERSION': self.version.linux_upstream_full,
             'ABINAME': self.abiname,
             'SOURCEVERSION': self.version.complete,
         })
@@ -182,7 +191,7 @@ class Gencontrol(Base):
         vars: dict[str, str],
         makeflags: MakeFlags,
     ) -> None:
-        makeflags['KERNEL_ARCH'] = config.name_kernelarch
+        makeflags['KERNEL_ARCH'] = vars['kernel_arch'] = config.name_kernelarch
 
     def do_arch_packages(
         self,
@@ -282,27 +291,17 @@ linux-signed-{vars['arch']} (@signedtemplate_sourceversion@) {dist}; urgency={ur
         do_meta = config.packages.meta
 
         relation_compiler = PackageRelationEntry(cast(str, config.build.compiler))
+        relation_compiler_host = PackageRelationEntry(
+            relation_compiler,
+            name=f'{relation_compiler.name}-for-host',
+        )
 
-        relation_compiler_header = PackageRelationGroup([relation_compiler])
-
-        # Generate compiler build-depends for native:
-        # gcc-N [arm64] <!cross !pkg.linux.nokernel>
+        # Generate compiler build-depends:
         self.bundle.source.build_depends_arch.merge([
             PackageRelationEntry(
-                relation_compiler,
+                relation_compiler_host,
                 arches={arch},
-                restrictions='<!cross !pkg.linux.nokernel>',
-            )
-        ])
-
-        # Generate compiler build-depends for cross:
-        # gcc-N-aarch64-linux-gnu [arm64] <cross !pkg.linux.nokernel>
-        self.bundle.source.build_depends_arch.merge([
-            PackageRelationEntry(
-                relation_compiler,
-                name=f'{relation_compiler.name}-{config.defs_debianarch.gnutype_package}',
-                arches={arch},
-                restrictions='<cross !pkg.linux.nokernel>',
+                restrictions='<!pkg.linux.nokernel>',
             )
         ])
 
@@ -388,7 +387,7 @@ linux-signed-{vars['arch']} (@signedtemplate_sourceversion@) {dist}; urgency={ur
                     desc.append(config.description.long[part])
                     desc.append_short(config.description.short[part])
 
-        packages_headers[0].depends.merge(relation_compiler_header)
+        packages_headers[0].depends.merge([relation_compiler_host])
         packages_own.extend(packages_image)
         packages_own.extend(packages_headers)
 
@@ -438,8 +437,11 @@ linux-signed-{vars['arch']} (@signedtemplate_sourceversion@) {dist}; urgency={ur
                 self.bundle.add('image-extra-dev', ruleid, makeflags, vars, arch=arch)
             )
 
-        # In a quick build, only build the quick flavour (if any).
-        if not config.defs_flavour.is_quick:
+        # In a quick build, only build the test flavour.
+        if config.defs_flavour.is_test:
+            for package in packages_own:
+                package.build_profiles[0].pos.add('pkg.linux.quick')
+        else:
             for package in packages_own:
                 package.build_profiles[0].neg.add('pkg.linux.quick')
 
@@ -462,7 +464,7 @@ linux-signed-{vars['arch']} (@signedtemplate_sourceversion@) {dist}; urgency={ur
         self.tests_control.extend(tests_control_headers)
 
         kconfig = []
-        for c in config.config:
+        for c in (config.config_nodefault if config.defs_flavour.is_test else config.config):
             for d in self.config_dirs:
                 if (f := d / c).exists():
                     kconfig.append(str(f))
@@ -556,17 +558,25 @@ linux-signed-{vars['arch']} (@signedtemplate_sourceversion@) {dist}; urgency={ur
     def process_changelog(self) -> None:
         version = self.version = self.changelog[0].version
 
-        if self.changelog[0].distribution == 'UNRELEASED':
-            self.abiname = f'{version.linux_upstream}+unreleased'
-        elif self.changelog[0].distribution == 'experimental':
-            self.abiname = f'{version.linux_upstream}'
-        elif version.linux_revision_backports:
-            self.abiname = f'{version.linux_upstream_full}+bpo'
+        if self.debianrelease.abi_version_full:
+            self.abiname = version.linux_upstream_full \
+                + self.debianrelease.abi_suffix
+            # All Debian versions must have a distinct ABI version.
+            # So if this is not the first Debian version with its
+            # upstream version and Debian release, distinguish it by
+            # adding a serial number suffix.
+            n = sum(1
+                    for entry in self.changelog
+                    if (entry.version.linux_upstream_full == version.linux_upstream_full
+                        and self.debianrelease.name_regex.fullmatch(entry.distribution)))
+            if n > 1:
+                self.abiname += f'+{n-1}'
         else:
-            self.abiname = f'{version.linux_upstream_full}'
+            self.abiname = version.linux_version_update \
+                + self.debianrelease.abi_suffix
 
         self.vars = {
-            'upstreamversion': self.version.linux_upstream,
+            'upstreamversion': self.version.linux_upstream_full,
             'version': self.version.linux_version,
             'version_complete': self.version.complete,
             'source_basename': re.sub(r'-[\d.]+$', '',
@@ -578,26 +588,9 @@ linux-signed-{vars['arch']} (@signedtemplate_sourceversion@) {dist}; urgency={ur
         self.vars['source_suffix'] = \
             self.changelog[0].source[len(self.vars['source_basename']):]
 
-        distribution = self.changelog[0].distribution
-        if distribution in ('unstable', ):
-            if version.linux_revision_experimental or \
-               version.linux_revision_backports or \
-               version.linux_revision_other:
-                raise RuntimeError("Can't upload to %s with a version of %s" %
-                                   (distribution, version))
-        if distribution in ('experimental', ):
-            if not version.linux_revision_experimental:
-                raise RuntimeError("Can't upload to %s with a version of %s" %
-                                   (distribution, version))
-        if distribution.endswith('-security') or distribution.endswith('-lts'):
-            if version.linux_revision_backports or \
-               version.linux_revision_other:
-                raise RuntimeError("Can't upload to %s with a version of %s" %
-                                   (distribution, version))
-        if distribution.endswith('-backports'):
-            if not version.linux_revision_backports:
-                raise RuntimeError("Can't upload to %s with a version of %s" %
-                                   (distribution, version))
+        if not self.debianrelease.revision_regex.fullmatch(version.revision):
+            raise RuntimeError(
+                f"Can't upload to {self.changelog[0].distribution} with a version of {version}")
 
     def write(self) -> None:
         super().write()
